#!/usr/local/bin/tclsh
# Generates classIndex.gen.html (class index)
# moduleClass-dir.gen.html
puts "Sourcing doc-all-classes.gen.tcl"
source doc-all-classes.gen.tcl
puts "Sourced doc-all-classes.gen.tcl"
########################################################################
#                   Finding HTML versions of class docs
########################################################################
proc genIndexFile { classes } {
    global gClassDef

    set  indexfl [open "classIndex.gen.html" w]
    puts $indexfl "<H1><a href=\"http://www.icsi.berkeley.edu/Sather\"><iMG SRC=\"campanile_smaller.gif\"></a>Alphabetic Class Index</H1>"
#    puts $indexfl "<h1> Alphabetic Class Index </h1>"
    puts $indexfl "Generated by genTopHTMLdoc.tcl. "
    puts $indexfl  "Contact Benedict Gomes <a href=\"mailto:gomes@icsi.berkeley.edu\">gomes@icsi.berkeley.edu</a>"
    puts $indexfl  "<li> <a href=moduleClass-dir.gen.html><b>Module/Class listing</b></a>"
    set sortedClasses [lsort -increasing -ascii $classes ]
    puts $indexfl "<dir>"
    foreach class ${sortedClasses} {
	set cdef  $gClassDef(${class})
	set classFileName [lindex ${cdef} 0]
	set classFileOnly [getFileName ${classFileName}]
	set classLineOffset [lindex ${cdef} 1] 
	puts $indexfl "<li><a href=\"lined-${classFileOnly}.gen.html#Line${classLineOffset}\"><b>${class}</b></a> \t   ${classFileName}  Line: ${classLineOffset}"

    }
    puts $indexfl "</dir>"
    close $indexfl
}

proc genTopLevelHTML { } {
	# Loop over all classes and dump the
        # associated comments to a file...
    
    global gModToClasses
    global gClassDef

    puts "starting to generate"
    set mainfl [open "moduleClass-dir.gen.html" w]
    puts "here"
    puts $mainfl "<body background=\"bluestone.gif\" >"
    puts $mainfl "<H1><a href=\"http://www.icsi.berkeley.edu/Sather\"><iMG SRC=\"campanile_smaller.gif\"></a>Module And Class Directory</H1>"
    puts $mainfl "Generated by genTopHTMLdoc.tcl."
    puts $mainfl  "Contact Benedict Gomes <a href=\"mailto:gomes@icsi.berkeley.edu\">gomes@icsi.berkeley.edu</a> for more information"
    #    puts $mainfl "
    puts $mainfl "<li> <a href=classIndex.gen.html><b>Class Index</b></a>"
    puts $mainfl "<li> <a href=http://www.icsi.berkeley.edu/~gomes/Sather/sather-top.html><b>Ben's Home Page</b></a>"
    puts "Done header"
    set classes [array names gClassDef]
    puts "All classes: ${classes}. Generating index file...."
    genIndexFile ${classes}
    puts "Done Generating index file...."
    set unsortedModules [array names gModToClasses]
    set sortedModules [lsort -increasing -ascii $unsortedModules ]
    puts $mainfl "<h2> Modules </h2>"
    puts $mainfl "<dir>"
    foreach module ${sortedModules} {
	set textNameForModule [getModuleName ${module}]
	puts $mainfl "<li><a href=#${textNameForModule}>${textNameForModule}</a>"
    }
    puts $mainfl "</dir>"
    puts "Sorted modules: ${sortedModules}"
    foreach module $sortedModules {
	puts "Dealing with module: $module"
	set textNameForModule [getModuleName ${module}]
	set modComment [getNextMultiLine ${module} 0]
	#	puts $mainfl "<dt><a href=\"${textNameForModule}\"><b>${textNameForModule}</b></a> ${module}"
	puts $mainfl "<a name=\"${textNameForModule}\"><h3>${textNameForModule} </h3></a>(${module})"
	puts $mainfl "<pre><em> ${modComment} </em></pre>"
	set classes [lsort -increasing -ascii $gModToClasses($module)]
	puts $mainfl "<pre>"
	foreach cl ${classes} {
	    puts "Processing ${cl}..."
	    set classNm ${cl}
	    if { [isAbstract ${classNm}]} {
		set classNm [string range ${classNm} 1 end]
		set cdef $gClassDef(${classNm})
	    } else {
		set cdef $gClassDef(${classNm})
	    }
	    set features [lindex $cdef 2]
	    set classFileName [lindex ${cdef} 0]
	    set classFileOnly [getFileName ${classFileName}]
	    set classLineOffset [lindex ${cdef} 1] 
	    # puts "Class def: ${cdef}"
	    set classComment \
		    [getCommentFollowing $classFileName [expr $classLineOffset - 1]]
	    set classComment [cleanComment $classComment]
	    puts $mainfl "   <a href=\"lined-${classFileOnly}.gen.html#Line${classLineOffset}\">${classNm}</a>"
	    puts $mainfl "<em>${classComment}</em>"
	}
	puts $mainfl "</pre>"
    }
    puts "Done with modules"
    close $mainfl
}

proc cleanComment { commentLines } {
    regsub -all -- {--} ${commentLines} "       " retVal
    return ${retVal}
}

proc getModuleName { fullName } {
    #  puts "Getting module name:$fullName"
    set match [regexp {[A-Za-a_/0-9]*/([A-Za-z_0-9]*).(module|com)} $fullName full modname ]
    if { $match == 1 } {
	return $modname
    } else {
	set match [regexp {([A-Za-z_0-9]*).(module|com)} $fullName full modname ]
	if { $match == 1} {
	    return $modname
	} else {
	    return ""
	    #mbk
	}
    }
}

# Return the comment that follows a feature or a class definition
# i.e. go through lines until you find one without a -- or an "include" in it.
proc getCommentFollowing { fileName routineLoc } {
    set fileText [getAllFile $fileName]
    set lines [split $fileText "\n"]
    set firstLine [lindex $lines $routineLoc]
    set isCom [string first "--" $firstLine]
    if { $isCom >= 0 } {
	set l [string length ${firstLine}]
	set nl [string range ${firstLine} $isCom $l]
	set retLines "${nl} \n"
    } else {
	set retLines ""
    }
    incr routineLoc
    set nextLine [string trimleft [lindex $lines $routineLoc]]
    set isCom [string first "--" $nextLine]
    set isInc [string first "include" $nextLine]
    #    puts "Next line: $nextLine $isCom $isInc"
    while { [expr (($isCom == 0) || ($isInc >= 0))] } {
	incr routineLoc
	if { $isCom == 0 } {
	    set retLines "$retLines$nextLine \n"
	}
	set nextLine [string trimleft [lindex $lines $routineLoc]]
	set isCom [string first "--" $nextLine]
	set isInc [string first "include" $nextLine]
    }
    return $retLines

}


# Return the comment that follows a feature or a class definition
# i.e. go through lines until you find one without a -- or an "include" in it.
proc getNextMultiLine { fileName routineLoc } {
    set fileText [getAllFile $fileName]
    set commentFound [regexp {\(\*(.*)\*\)} ${fileText} fullMatch comment]
    if {$commentFound} {
	return ${comment}
    } else { 
	puts "No module comment found"
	return ""
    }
}

#Return the last part of a sather filename from a full  path
proc getFileName { fullName } {
    # puts "Getting file name: ${fullName}"
    set match [regexp {[A-Za-a_/0-9]*/([A-Za-z_0-9]*.sa)} $fullName full modname ]
    if { $match == 1 } {
	return $modname
    } else {
	set match [regexp {([A-Za-z_0-9]*).module} $fullName full modname ]
	return $modname
    }
}

proc getAllFile { fname } {
    set ret ""
    if { [file exists ${fname}] } {
	set f [open ${fname} r]
	set ret [read $f]
	close $f
    } else {
	puts "No Description file: $fname"
	set ret "No Description file."
    }
    return $ret
}

proc isAbstract { nodeName } {
    set abs [string first "\$" ${nodeName}]
    # possible to return abs != -1 itself?
    if { $abs != -1 } {
	return true
    } else {
	return false
    }
    
}

genTopLevelHTML

#proc genClassDoc { } {
#	    # Generating doc file
#	    set referClass "<a href=\"lined-${classFileOnly}.html#Line${classLineoffset}\"><b> ${classNm} </b></a>"
#	    set clname "doc-${cl}.html"
#	    set fl [open $clname w]
#	    if {[isAbstract ${classNm}] } {
#		puts $fl "<pre>\ntype ${referClass} is \n$classComment \n"
#	    } else {
#		puts $fl "<pre>\nclass ${referClass} is \n$classComment \n"
#	    }
#	    set sortedRouts [lsort -increasing -ascii ${features}]
#	    puts $fl "<dl>"
#	    foreach rout ${sortedRouts} {
#		# Must be changed....
#		set featureSig  [lindex ${rout} 0]	
#		set featureFileName [lindex ${rout} 1]
#		set featureFileOffset [lindex ${rout} 2]
#		set featureFileOnly [getFileName ${featureFileName}]
#		puts $fl "<dt><a href=\"lined-${featureFileOnly}.html#Line${featureFileOffset}\"><b>${featureSig}</b></a>"
#		set com [getCommentFollowing $featureFileName \
#			[expr $featureFileOffset - 1]]
#		set com [cleanComment ${com}]
#		puts $fl "<dd>${com}"
#	    } 
#	    puts $fl "</dl>"
#	    close $fl
#	}
#	# End of a module
#	puts $mainfl "</dl>\n"
#    }
#    close $mainfl
#}

