;;; -*-emacs-lisp-*-

;;; This file contains some convenient forms you may wish to put in
;;; one of your Emacs startup files.  On the other hand, you can just
;;; uncomment the things you want and load the whole file from your
;;; `.emacs' file, like this:
;;;
;;; (load "sample.emacs")

;; Complain if SATHER_HOME is not defined (temp addition by Ben)
(if (getenv "SATHER_HOME")
    ()
  (error "Environment variable SATHER_HOME is not defined!")
)


;;; Look for the ELisp files included in the Sather distribution.
(setq load-path
      (append
       (list (expand-file-name (concat (getenv "SATHER_HOME") "/Emacs")))
       load-path))

;;; Look forthe Info files included in the Sather distribution.
(setq Info-directory-list
      (append
       (list (expand-file-name (concat (getenv "SATHER_HOME") "/Emacs")))
       Info-default-directory-list))

;;; This is how Emacs knows how to load Sather source files.
(autoload 'sather-mode "sather" "Sather Mode" t)
(setq auto-mode-alist
      (append '(("\\.sa$" . sather-mode)
		) auto-mode-alist))

; (load-file (expand-file-name (concat (getenv "SATHER_HOME") "/Emacs/hl319.elc")))

;;; The next three code segements set up different packages that
;;; support syntax highlighting for program source code.  sather-mode
;;; supports all three.  Which one to choose is up to you; however, I
;;; would strongly suggest that you use hl319 if you are using GNU
;;; Emacs 19, and font-lock if you are using XEmacs 19.

;; This sets up the hl319 package for syntax highlighting.  If you use
;; a light-colored background (like white), change `dark' to `light'
;; for `hilit-background-mode'.  You may choose others colors that suit
;; you.  I found that the font size, monitor, and personal preference
;; makes this a highly individual choice.
(cond (window-system
       (add-hook 'find-file-hooks    'hilit-install-line-hooks)
       (setq hilit-mode-enable-list  '(not text-mode)
             hilit-background-mode   'dark)
       (setq hilit-user-face-table
             ;; face         light      dark            mono
             '((comment      nil	pink-italic  	nil)
               (default      nil	wheat           nil)
               (constant     nil 	sandybrown      nil)
               (include      nil	nil             nil)
               (jargon-entry nil	nil             nil)
               (string       nil	sandybrown      nil)
               (sugar        nil	white         	nil)
               (iterator     green-bold cyan            bold)
               (keyword      green-bold white           bold)
               (builtin      white-bold wheat-bold      nil)
               (type         nil	wheat-bold 	bold)
	       (exception    nil        nil             nil)
	       (defun        nil	white-bold	bold)
               ))
       (require 'hl319)
       ))

;; This sets up the hilit19 package for syntax highlighting.
;(cond (window-system
;       (setq hilit-mode-enable-list  '(not text-mode)
;             hilit-background-mode   'dark
;             hilit-inhibit-hooks     nil
;             hilit-inhibit-rebinding nil)
;       (require 'hilit19)
;       (hilit-translate sugar    'Plum1
;			builtin  'white-bold
;			iterator 'green-bold)
;       ))

;; This sets up the font-lock package for syntax highlighting.
;(cond (window-system
;       (require 'font-lock)
;       ))

;;; This is what Emacs does when it loads a Sather file.
(setq sather-mode-hook
      '(lambda ()
	 ;; Set the standard indentation width to be three spaces (default).
	 ;(setq sather-indent 3)
	 ;; Turn on auto-indentation.
	 ;(sather-toggle-indent-state 1)
	 (setq sather-auto-indent t)
	 ;; Turn off auto-newlines (defualt).
	 ;(sather-toggle-newline-state -1)
	 ;; Turn on the ``hungry'' delete key.
	 ;(sather-toggle-hungry-state 1)
	 ;; Turn on the indent-prefix state (default).
	 ;(sather-toggle-prefix-state 1)
	 ;; Use font-lock for Sather buffers.
	 ;(font-lock-mode 1)
	 ;; Use the Sather Emacs Library functions.
	 (require 'sather-lib)
	 ;; Use the `backspace' key for deleting left.
	 ;(local-set-key [backspace] 'sather-electric-delete)
	 ;; Move forward a Sather block.
	 (local-set-key [M-S-right] 'sather-forward-sexp)
	 ;; Use a different function for deleting.
	 ;(setq sather-delete-function 'backward-delete-char)
	 ;; Use a good, standard tab width of 8.
	 (setq tab-width 8)
	 ;; Make Sather words include `_', `!', `#', and `$'.
	 (modify-syntax-entry ?_ "w")
	 (modify-syntax-entry ?! "w")
	 (modify-syntax-entry ?# "w")
	 (modify-syntax-entry ?$ "w")
	 ))

; This is for using the browser with emacs - it activates
; a function in sather-lib.el
(setq server-switch-hook
      '(lambda ()
         (require 'sather-lib)
         (if (string-match "\\.sa$" (buffer-file-name))
             (sather-server-buffer-undone (current-buffer)))
         ))
