#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/file.h>
#include <fcntl.h>
#include <stdio.h>
#include <memory.h>
#define RUNTIME
#include "sather.h"
#include "tags.h"

/* Set (or clear) a bit.  It's a fatal error for bit to be less than
 * zero or greater than or equal to maxbits.  This implementation
 * is probably endian dependent.  If anyone knows a more portable
 * way to do this, let me know.
*/

void rt_aset(void *buf, INT bit, BOOL val, INT maxbits)
{
    INT i,j,mask,*p;
    if (bit<0||bit>=maxbits) FATAL("aset index out of range");
    i = bit/(sizeof(INT)*8);
    j = bit%(sizeof(INT)*8);
    mask = ~(1<<j);
    p = ((INT*) buf)+i;
    *p = ((*p)&mask)|(val<<j);
}

BOOL rt_aget(void *buf, INT bit, INT maxbits)
{
    INT i,j,*p;
    if (bit<0||bit>=maxbits) FATAL("aget index out of range");
    i = bit/(sizeof(INT)*8);
    j = bit%(sizeof(INT)*8);
    p = ((INT*) buf)+i;
    return (BOOL) (((*p)&(1<<j))!=0);
}

FLT rt_flt_aset(FLT f,INT bit,BOOL val)
{
  rt_aset(&f,bit,val,32); 
  return f;
}

BOOL rt_flt_aget(FLT f, INT bit) 
{ 
  return rt_aget(&f, bit, 32);
}

INT rt_flt_str_in_prec(FLT f, INT prec, char *store_in)
{
  char buf[20];
  sprintf(buf,"%%.%dg",prec);
  sprintf(store_in,buf,f);
  return(strlen(store_in));
}

FLTD rt_fltd_aset(FLTD f, INT bit, BOOL val)
{
  rt_aset(&f, bit, val, 64);
  return f;
}
 
BOOL rt_fltd_aget(FLTD f, INT bit)
{
  return rt_aget(&f, bit, 64);
}

int rt_fltd_str_in_prec(FLTD f, INT precision, char* store_in) {
  char buf[20];
  sprintf(buf,"%%.%dg",precision);
  sprintf(store_in,buf,f);
  return(strlen(store_in));
}

/* Open a file for reading. */
int rt_str_file_open(char *nm){return open(nm,O_RDONLY,0);}

/* Return the size of the file with descriptor `fd'. */
int rt_str_file_size(int fd){return (int)lseek(fd,0L,2);}

/* Fill in `s' with the characters of `fd' starting at `st' and
going for `sz' chars. */
void rt_str_file_in_str(int fd, char *s, int st, int sz)
{lseek(fd,st,0); read(fd,s,sz);}

/* Fill in `s' with the characters of `fd' starting at `st' and
going for `sz' chars, start at an offset of `bst' in `s'. */
void rt_str_file_in_fstr(int fd, char *s, int st, int sz, int bst)
{lseek(fd,st,0); read(fd,s+bst,sz);}

/* Close the file described by descriptor `fd'. */
void rt_str_file_close(int fd){close(fd);}

/* Split concatinate strings separated by '\0' into array of string in C. */
char **
rt_create_astr (int size, char *s)
{
  char **ptr;
  int i;

  ptr = (char **) malloc (sizeof (char *) * size);
  for (i = 0; (i < size) && (*s != 0); i++)
    {
      ptr[i] = s;
      s = (char *) (s + strlen(s) + 1);
    }
  return  ptr;
}

/* For exception handling, a global storing the state information
 * needed to longjmp back to the dynamically most recent protect
 * statement.  Also needed is a global for returning the exception
 * value.
*/

jmp_buf last_protect;
void *exception;

static INT ob_count;

void *rt_alloc(size_t size, INT tag) {
    void *res;
    /* printf("sbi_alloc %d %d\n",size,tag); */
    res=GC_malloc(size);
    if (res==NULL) FATAL("Unable to allocate more memory");
    ((OB)res)->header.tag=tag;
#ifdef DETERMINISTIC
    ((OB)res)->header.id=ob_count++;
#endif
    return res;
}

void *rt_arr_alloc(size_t size1, INT tag, size_t size2, INT n) {
    void *res;
    /* printf("sbi_arr_alloc %d %d %d %d\n",size1,tag,size2,n); */
    res=GC_malloc_ignore_off_page(size1+(n-1)*size2);
    if (res==NULL) FATAL("Unable to allocate more memory");
    ((OB)res)->header.tag=tag;
#ifdef DETERMINISTIC
    ((OB)res)->header.id=ob_count++;
#endif
    return res;
}

void *rt_alloc_atomic(size_t size, INT tag) {
     void *res;
     /* printf("sbi_alloc_atomic %d %d\n",size,tag); */
     res=GC_malloc_atomic(size);
     /* we must clear it manually */
     if (res==NULL) FATAL("Unable to allocate more memory");
     memset(res,0,size);
     ((OB)res)->header.tag=tag;
#ifdef DETERMINISTIC
     ((OB)res)->header.id=ob_count++;
#endif
     return res;
}
 
void *rt_arr_alloc_atomic(size_t size1, INT tag, size_t size2, INT n) {
     void *res;
     /* printf("sbi_arr_alloc %d %d %d %d\n",size1,tag,size2,n); */
     size_t size = size1+(n-1)*size2;
     /* Allocate one more byte than necessary, to ensure that
     *  strings are always null terminated.
     */
     res=GC_malloc_atomic_ignore_off_page(size+1);
     if (res==NULL) FATAL("Unable to allocate more memory");
     memset(res,0,size);
     ((OB)res)->header.tag=tag;
#ifdef DETERMINISTIC
     ((OB)res)->header.id=ob_count++;
#endif
     return res;
}
 

/* This catches void references */

void rt_segfault_handler()
{
  FATAL("Attribute or array access of void");
}

/* This isn't declared void so we won't get warnings when we
 * "use" it's result in-line during arithmetic and bounds checks
*/
int rt_fatal(char *file, int line, char *msg) {
  fprintf(stderr,"%s:%d: Runtime error - %s\n",file,line,msg);
  fflush(stderr);
#ifdef PSATHER
  PSATHER_STOP;
#endif
  abort();
}
/* rt_fatal2 is used when -debug wasn't given so there's no
 * symbolic info to print
*/
int rt_fatal2(char *msg) {
  fprintf(stderr,"Runtime error - %s\n",msg);
  fprintf(stderr,"(Use \"-debug\" to compile symbolic info)\n");
  fflush(stderr);
#ifdef PSATHER
  PSATHER_STOP;
#endif
  abort();
}

/* Statistics collection */

#ifdef STATS
int rt_dispatches=0;
time_t time_at_start;
#endif

#ifdef PSATHER
struct CLEARED_EX_struct rt_cleared_ex;
#endif

void rt_start(char *executable) {
#ifdef PSATHER
    PSATHER_START(executable);
    rt_cleared_ex.header.tag = CLEARED_EX_tag;
    SET_CLEAR_EXCEPTION(&rt_cleared_ex);;
#endif
#ifdef STATS
    time_at_start=time(NULL);
#endif
}
void rt_stop() {
#ifdef STATS
    fprintf(stderr,"\nRuntime statistics:\n\n");
    fprintf(stderr,"    Dispatched calls:      %d\n",rt_dispatches);
    fprintf(stderr,"    Number of collections: %d\n",GC_gc_no);
    GC_gcollect();
    fprintf(stderr,"    Live heap at end:      %d Kb\n",
	                                   GC_get_heap_size()/1024);
    fprintf(stderr,"    Execution wall time:   %d secs\n\n",
	                                   time(NULL)-time_at_start);
#endif
#ifdef PSATHER
    PSATHER_STOP;
#endif
}

/* This is from stoehr@informatik.tu-muenchen.de */
#if defined(__hpux) || defined(linux)
double scalbn(double v, int n)
{
    int i,nn;
    double res;

    res = 1.0;
    nn = abs(n);
    for (i=0; i<nn; i++)
        res += res;
    if (n<0) return(v / res);
    return(v*res);
}
#endif
#if defined (hpux)
char *getwd(char *a)
{
    return(getcwd(a,1023));
}
#endif

/* Open a file for reading. */
int rt_file_open(char *nm){return open(nm,O_RDONLY,0);}

/* Return the size of the file with descriptor `fd'. */
int rt_file_size(int fd){return (int)lseek(fd,0L,2);}

/* Fill in `s' with the characters of `fd' starting at `st' and
going for `sz' chars. */
void rt_file_in_str(int fd, char *s, int st, int sz)
{lseek(fd,st,0); read(fd,s,sz);}

/* Fill in `s' with the characters of `fd' starting at `st' and
going for `sz' chars, start at an offset of `bst' in `s'. */
void rt_file_in_fstr(int fd, char *s, int st, int sz, int bst)
{lseek(fd,st,0); read(fd,s+bst,sz);}

/* Close the file described by descriptor `fd'. */
void rt_file_close(int fd){close(fd);}

/* for RISCos5.01 BSD43, added by yoshida@agusa.nuie.nagoya-u.ac.jp*/
#if defined(V_RISCOS) && defined(R_5_0)
# ifdef feof
#  undef feof
# endif
# ifdef ferror
#  undef ferror
# endif

int feof(FILE *p) { return (((p)->_flag&_IOEOF)!=0); }
int ferror(FILE *p) { return (((p)->_flag&_IOERR)!=0); }
#endif
